# Test script for TagIT.
# This script is runnable with `make test' or `perl 1.t'

use strict;
use warnings;
use Cwd;
use Shell;
use Test::More tests => 17;

# Setup the test directory.
if (-e 'test' && -d 'test') {
    # Make sure the test directory does not already exist.
    &BAIL_OUT ("\nThe test directory 'test/' already exists.\n" .
               "Please remove or rename the directory.");
} elsif (-e 'test') {
    &BAIL_OUT ("\nCould not create the test directory 'test/'.\n" .
               "A file named 'test' already exists.");
} else {
    # Make a directory to hold the test files.
    mkdir ('test') or &BAIL_OUT ("Could not create test directory: $!");
}

# Locate the tagit program.
my $PROG = 'tagit';
if (-x './tagit') {
    $PROG = './tagit';
} elsif (-x '../tagit') {
    $PROG = '../tagit';
}

# We will use a test database for these test.
# No sense in polluting the users database.
$PROG .= ' -Z test/test.db';

# Test #1
# Make sure the program can run by getting part of the version.
{
    my $result = `$PROG -v`;
    $result =~ s/^(\w+:)\s+.*\n$/$1/;
    unless (&is ($result, 'tagit:', 'Execution check')) {
        &clean_up;
        &BAIL_OUT ("\nCould not execute tagit.\n" .
                   "I looked in ./ and ../ and \$PATH.");
    }
}

# Test #2
# Add a file.
{
    &touch ('test/test.tst');
    my $result = `$PROG test/test.tst TagIT Test`;
    $result =~ s/\n//g;
    my $test = 'tagit: test.tst -> TagIT, Testtagit: The file has been added.';
    &is ($result, $test, 'Add a file');
}

# Test #3
# Add a tag.
{
    my $result = `$PROG test/test.tst New_Tag`;
    $result =~ s/\n//g;
    my $test = 'tagit: The file has been updated.';
    &is ($result, $test, 'Add a tag');
}

# Test #4
# Remove a tag.
{
    my $result = `$PROG -d test/test.tst New_Tag`;
    $result =~ s/\n//g;
    my $test = 'tagit: The tag(s) have been deleted from the file.';
    &is ($result, $test, 'Remove a tag (-r)');
}

# Test #5
# Replace the tags on a file.
{
    my $result = `$PROG -R test/test.tst New_Tag Test`;
    $result =~ s/\n//g;
    my $test = 'tagit: The tag(s) have been replaced.';
    &is ($result, $test, 'Replace the tags (-R)');
}

# Test #6
# Delete a tag from the database.
{
    my $result = `$PROG -D New_Tag`;
    $result =~ s/\n//g;
    my $test = 'tagit: The tag(s) have been deleted.';
    &is ($result, $test, 'Delete a tag (-D)');
}

# Test #7
# List tags (-t).
{
    my $result = `$PROG -t`;
    $result =~ s/\n//g;
    my $test = 'TagIT -> 0Test -> 1--2 tags';
    &is ($result, $test, 'List tags (-t)');
}

# Test #8
# List tags (-T).
{
    my $result = `$PROG -T`;
    $result =~ s/\n//g;
    my $test = 'Test -> 1TagIT -> 0--2 tags';
    &is ($result, $test, 'List tags (-T)');
}

# Test #9
# List files (-l).
{
    my $result = `$PROG -l`;
    $result =~ s/\n//g;
    my $test = 'test.tst -> Test--1 files';
    &is ($result, $test, 'List files (-l)');
}

# Test #10
# List files with ID (-I -l).
{
    my $result = `$PROG -Il`;
    $result =~ s/\n//g;
    my $test = 'AAAA test.tst -> Test--1 files';
    &is ($result, $test, 'List files with ID (-I -l)');
}

# Test #11
# Add a note (-n).
{
    my $result = `$PROG -n 'This is a note.' test/test.tst`;
    $result =~ s/\n//g;
    my $test = 'tagit: Note added.';
    &is ($result, $test, 'Add a note (-n)');
}

# Test #12
# Delete a note (-N).
{
    my $result = `$PROG -N test/test.tst`;
    $result =~ s/\n//g;
    my $test = 'tagit: Note deleted.';
    &is ($result, $test, 'Delete a note (-N)');
}

# Test #13
# Reassign a file by ID quietly (-F -i -q).
{
    unlink ('test/test.tst');
    &touch ('test/test.new');
    my $result = `$PROG -q -F test/test.new -i AAAA`;
    $result =~ s/\n//g;
    my $test = '';
    &is ($result, $test, 'Reassign a file by ID quietly (-F -i -q)');
}

# Test #14
# List files with path (-l -p).
{
    my $result = `$PROG -l -p`;
    $result =~ s/\n//g;
    my $dir = &getcwd;
    my $test = $dir . '/test/test.new -> Test--1 files';
    &is ($result, $test, 'List files with path (-l -p)');
}

# Test #15
# List file file by name (-f).
{
    my $result = `$PROG -f test/test.new`;
    $result =~ s/\n//g;
    my $test = 'test.new -> Test';
    &is ($result, $test, 'List file file by name (-f)');
}

# Test #16
# Remove a file from the database (-x).
{
    my $result = `$PROG -x test/test.new`;
    $result =~ s/\n//g;
    my $test = 'test.new -> Testtagit: The file has been deleted.';
    &is ($result, $test, 'Remove a file from the database (-x)');
}

# Test #17
# Update the database (-u).
{
    my $result = `$PROG -u`;
    $result =~ s/\n//g;
    my $test = 'tagit: Database update completed. 0 files updated.';
    &is ($result, $test, 'Update the database (-u)');
}

# Last thing to do, clean up after ourselves.
&clean_up;

# Remove the test files.
sub clean_up {
    unlink ('test/test.db');
    unlink ('test/test.lck');
    unlink ('test/test.tst');
    unlink ('test/test.new');
    rmdir ('test');
}
